package com.starbuds.server.game.sdkdev;

import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.digest.MD5;
import cn.hutool.http.Header;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;

/**
 * @author aoe
 * @date 2020/9/7
 */
@Slf4j
public class DefaultGameDevClient implements GameDevClient {

    /**
     * 主机地址
     */
    private String host;

    /**
     * 直播平台id
     */
    private String appId;

    /**
     * API秘钥
     */
    private String apiSecretKey;

    /**
     * HTTP超时时间（毫秒）
     */
    private int timeout;

    private DateTimeFormatter dtf;

    public DefaultGameDevClient(String host, String appId, String apiSecretKey, Integer timeout) {
        this.host = host;
        this.appId = appId;
        this.apiSecretKey = apiSecretKey;
        this.timeout = timeout == null ? 5000 : timeout;
        dtf = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
    }

    /**
     * 执行Http请求
     *
     * @param request
     * @return
     */
    @Override
    public <T extends GameDevResponse> T execute(GameDevRequest<T> request) {
        String traceId = IdUtil.fastSimpleUUID();
        log.info("{} Game 请求: {}", traceId, JSON.toJSONString(request));
        JSONObject param = request.getParam() == null ? new JSONObject()
                : JSON.parseObject(JSON.toJSONString(request.getParam()));
        param.put("appid", appId);
        // 北京时间
        long timestamp = System.currentTimeMillis();
        param.put("timestamp", "" + timestamp);
        param.put("sign", sign(timestamp, param.getString("method")));
        String url = getApiUrl(request);
        String requestBody = param.toJSONString();
        log.info("{} Game 请求参数: {} {}", traceId, url, requestBody);

        try {
            HttpResponse response = HttpRequest.post(url)
                    .header(Header.CONTENT_TYPE, "application/json;charset=utf-8;")
                    .body(requestBody)
                    .timeout(request.timeout() > 0 ? request.timeout() : timeout)
                    .execute();
            if (response.isOk()) {
                log.info("{} Game 请求结果: {}", traceId, response.body());
                return JSON.parseObject(response.body(), request.getResponseClass());
            }
        } catch (Exception e) {
            log.error("dev 请求失败 traceId=" + traceId, e);
        }
        String failJson = "{\"code\":-1,\"msg\":\"请求超时，请重试\",\"request_id\":\"4f966b99-e87f-48a3-822a-139a197b94f9\"}";
        return JSON.parseObject(failJson, request.getResponseClass());
    }

    private String sign(long timestamp, String method) {
        // 签名规则：md5(appid+timestamp+method+secret)  --默认小写
        String signParam = appId + timestamp + method + apiSecretKey;
        return MD5.create().digestHex(signParam).toLowerCase();
    }

    private String getApiUrl(GameDevRequest request) {
        return StrUtil.format("{}{}",
                host,
                request.getServiceName());
    }
}
